 /*******************************************************************/
 /* Licensed Materials - Property of IBM                            */
 /*                                                                 */
 /* SupportPac MQ01                                                 */
 /*                                                                 */
 /*  Copyright IBM Corporation 2006                                */
 /* All rights reserved                                             */
 /*                                                                 */
 /* US Government Users Restricted Rights - Use, duplication or     */
 /* disclosure restricted by GSA ADP Schedule Contract with         */
 /* IBM Corp.                                                       */
 /*******************************************************************/
 /*                                                                 */
 /*******************************************************************/
 /* Issue an MQCONNX call with SSL and LDAP setup                   */ 
 /*                                                                 */
 /* Example syntax:                                                 */
 /*    sslput  -m Qmgr -k Repos -c Conn [-n Chl] [-s Spec]          */
 /*           [-q Queue] [-l ldapserver [-u User -p Pwd]]           */
 /*                                                                 */
 /* where:  -m Qmgr is the name of the queue manager                */
 /*         -n Chl specifies the name of the channel to connect to  */
 /*         -k Repos specifies the stem name of the key repository  */
 /*         -c Conn specifies the WMQ connection name: host(port)   */
 /*                                                                 */
 /* optional:                                                       */
 /*         -q Queue specifies the target queue to put messages to  */
 /*         -s Spec specifies the name of the SSL cipherspec to use */
 /*         -l ldapserver specifies the host name and port for LDAP */
 /*         -u User specifies the user name for the LDAP server     */
 /*         -p Pwd specifies the password for the LDAP server       */
 /*                                                                 */
 /* Default values:                                                 */
 /*         -n SYSTEM.DEF.SVRCONN                                   */
 /*         -s NULL_MD5                                             */
 /*         -q SYSTEM.DEFAULT.LOCAL.QUEUE                           */
 /*                                                                 */
 /*******************************************************************/
 /* Compilation examples:                                           */
 /*                                                                 */
 /*   Windows:  cl /link:mqic.lib sslput.c                          */
 /*                                                                 */
 /*   UNIX:     cc -lmqic sslput.c -o sslput                        */
 /*                                                                 */
 /* Refer to the WebSphere MQ Application Programming Guide for     */
 /* more information on compiling WebSphere MQ client applications. */
 /* Also, check your platform Quick Beginnings Guide for any SSL    */
 /* issues specific to your environment.                            */
 /*                                                                 */
 /*******************************************************************/
 /* Suggested uses:                                                 */
 /*                                                                 */
 /*   (1) Verifying SSL certificate setup is correct                */
 /*                                                                 */
 /*   (2) Checking that revoked certificates do not allow SSL       */
 /*       connections to be established.                            */
 /*                                                                 */
 /*******************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>

 /******************************************************************/
 /*                                                                */
 /*   Include files for WebSphere MQ                               */
 /*                                                                */
 /******************************************************************/
 #include <cmqc.h>                  /* For regular MQI definitions   */
 #include <cmqxc.h>                 /* For MQCD definition           */

 /******************************************************************/
 /*                                                                */
 /*   Useful macros for this application                           */
 /*                                                                */
 /******************************************************************/
 #define OK 0
 #define FAIL 1

 #ifndef min
 #define min(a,b) ((a) < (b) ? (a) : (b))
 #endif

 /* Size limits for buffers */
 #define BUFFER_LENGTH          2048

 #define LDAP_CONNAME_LENGTH    48
 #define LDAP_USER_LENGTH       48
 #define LDAP_PASSWORD_LENGTH   48
 
 #define SSL_CIPHER_SPEC_LENGTH 32
 #define SSL_KEYREPOS_LENGTH    48

 /* Default values for parameters */
 #define CHANNEL_NAME_DEFAULT   "SYSTEM.DEF.SVRCONN"
 #define QUEUE_NAME_DEFAULT     "SYSTEM.DEFAULT.LOCAL.QUEUE"
 #define SSL_CIPHER_DEFAULT     "NULL_MD5"

 /* Macro for copying a string parameter into a structure without overflow.
    This macro ensures the name ends up blank padded. */
 #define COPYPARAM_BP(destpointer,sourcepointer,destlength)  \
    memset((destpointer), ' ', (destlength));             \
    strncpy((destpointer), (sourcepointer), min( (destlength), strlen((sourcepointer)) ) )

 /* A non-blankpadded version of the above macro */
 #define COPYPARAM(destpointer,sourcepointer,destlength)  \
    memset((destpointer), 0, (destlength));               \
    strncpy((destpointer), (sourcepointer), min( (destlength), strlen((sourcepointer))+1 ) )

 
 void usageError()
 {
   printf("Usage: sslput -m Qmgr -k Repos -c Conn [-n Chl] [-s Spec] [-q Queue] ");
   printf("[-l ldapserver [-u ldapuser -p ldappasswd]]\n\n");
   
   printf("Please read the source code for more information\n");
   exit(1);
 }

 int main(int argc, char **argv)
 {

   /*   Declare MQI structures needed                                */
   MQCNO    ConnOpt   = {MQCNO_DEFAULT};
                                    /* MQCONNX options               */

   MQAIR    AuthRec = {MQAIR_DEFAULT};
                                    
   MQCD     ClientConn = {MQCD_CLIENT_CONN_DEFAULT};
                                    /* Client connection channel     */
                                    /* definition                    */
                                    
   MQSCO    ConnSCO = {MQSCO_DEFAULT};

   /* SSL and LDAP parameters */
   MQCHAR  SSLCipher[SSL_CIPHER_SPEC_LENGTH];
   MQCHAR  SSLKeyRepos[SSL_KEYREPOS_LENGTH];
   MQCHAR  LDAPServer[LDAP_CONNAME_LENGTH];
   MQCHAR  LDAPUser[LDAP_USER_LENGTH];
   MQCHAR  LDAPPwd[LDAP_PASSWORD_LENGTH];

   /* Queue manager and channel parameters */
   MQCHAR  ChannelName[MQ_CHANNEL_NAME_LENGTH];
   MQCHAR  MQConname[MQ_CONN_NAME_LENGTH];

   MQCHAR  QMName[MQ_Q_MGR_NAME_LENGTH];
   MQCHAR  QName[MQ_Q_NAME_LENGTH];

   MQLONG   O_options = 0;          /* open options for MQOPEN       */

   MQHCONN  Hcon;                   /* connection handle             */

   MQLONG   CompCode;               /* completion code               */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONNX       */

   /* Flags relating to which parameters we were given on the command line */
   int      haveQMName = 0;
   int      haveQName = 0;
   int      haveMQConname = 0;
   int      haveChannelName = 0;
   int      haveLDAPServer = 0;
   int      haveLDAPUser = 0;
   int      haveLDAPPwd = 0;
   int      haveSSLKeyRepos = 0;
   int      haveSSLCipher = 0;

   int      i;                      /* used to iterate through argv  */

   /* Details relating to putting a message onto a queue */
   MQLONG   OpenOptions = MQOO_FAIL_IF_QUIESCING | MQOO_OUTPUT;
   
   MQOD     ObjDesc = {MQOD_DEFAULT};
   MQHOBJ   hQueue;

   MQMD     md = {MQMD_DEFAULT};
   MQPMO    pmo = {MQPMO_DEFAULT};
   
   char     buffer[BUFFER_LENGTH];
   MQLONG   bufSize = BUFFER_LENGTH;
   MQLONG   messlen  = 0;

   FILE    *fp = NULL;

   /******************************************************************/
   /*                                                                */
   /*   Parse the command line given                                 */
   /*                                                                */
   /******************************************************************/
   i = 1;
   while(i < argc)
   {
     if((strlen(argv[i]) < 2) || (argv[i][0] != '-'))
     {
       printf("Error: unrecognised flag: %s\n\n", argv[i]);
       usageError();
     }

     switch(argv[i][1])
     {
       case 'm':  /* Queue manager name */
         i++;
         COPYPARAM_BP(QMName, argv[i], MQ_Q_MGR_NAME_LENGTH);
         haveQMName = 1;
         break;

       case 'q':  /* Queue name */  
         i++;
         COPYPARAM_BP(QName, argv[i], MQ_Q_NAME_LENGTH);
         haveQName = 1;
         break;

       case 'k':  /* Key repository name */
         i++;
         COPYPARAM(SSLKeyRepos, argv[i], SSL_KEYREPOS_LENGTH);
         haveSSLKeyRepos = 1;
         break;

       case 'c':  /* Channel connection name (conname) */
         i++;
         COPYPARAM(MQConname, argv[i], MQ_CONN_NAME_LENGTH);
         haveMQConname = 1;
         break;

       case 'n':  /* Server channel name, e.g. SYSTEM.DEF.SVRCONN */  
         i++;
         COPYPARAM_BP(ChannelName, argv[i], MQ_CHANNEL_NAME_LENGTH);
         haveChannelName = 1;
         break;

       case 's':  /* SSL cipherspec name, e.g. NULL_MD5 */
         i++;
         COPYPARAM(SSLCipher, argv[i], SSL_CIPHER_SPEC_LENGTH);
         haveSSLCipher = 1;
         break;

       case 'l':  /* LDAP server name, if CRL checking is desired */  
         i++;
         COPYPARAM(LDAPServer, argv[i], LDAP_CONNAME_LENGTH);
         haveLDAPServer = 1;
         break;

       case 'u':  /* LDAP server user name */
         i++;
         COPYPARAM(LDAPUser, argv[i], LDAP_USER_LENGTH);
         haveLDAPUser = 1;
         break;

       case 'p':  /* LDAP server password */
         i++;
         COPYPARAM(LDAPPwd, argv[i], LDAP_PASSWORD_LENGTH);
         haveLDAPPwd = 1;
         break;

       default:   /* Unknown parameter */
         printf("Error: unknown parameter %s\n\n", argv[i]);
         usageError();
     }

     i++; /* Next argument */
   }

   if(!haveQMName || !haveMQConname || !haveSSLKeyRepos)
   {
     printf("Error: -m, -c and -k are mandatory.\n\n");
     usageError();
   }

   /* For any missing optional values, fill in defaults */
   if(!haveQName)
   {
     COPYPARAM(QName, QUEUE_NAME_DEFAULT, MQ_Q_NAME_LENGTH);
   }

   if(!haveSSLCipher)
   {
     COPYPARAM(SSLCipher, SSL_CIPHER_DEFAULT, SSL_CIPHER_SPEC_LENGTH);
   }

   if(!haveChannelName)
   {
     COPYPARAM(ChannelName, CHANNEL_NAME_DEFAULT, MQ_CHANNEL_NAME_LENGTH);
   }
   
   /******************************************************************/
   /*                                                                */
   /*   Connect to queue manager                                     */
   /*                                                                */
   /******************************************************************/

   ConnOpt.Version = MQCNO_CURRENT_VERSION;
   ClientConn.Version = MQCD_CURRENT_VERSION;
   ConnSCO.Version = MQSCO_CURRENT_VERSION;

   /* Set up MQCD structure in MQCNO: */
   strncpy( ClientConn.ChannelName, ChannelName, strlen(ChannelName) );
   strncpy( ClientConn.ConnectionName, MQConname, strlen(MQConname) );
   ClientConn.MaxMsgLength = bufSize * 2; /* Ensure channel can handle big enough messages */
   
   ConnOpt.ClientConnPtr = (MQPTR) &ClientConn;
   
   /* Fill in SSL cipherspec and repository */
   strncpy( ClientConn.SSLCipherSpec, SSLCipher, strlen(SSLCipher) );
   strncpy( ConnSCO.KeyRepository, SSLKeyRepos, strlen(SSLKeyRepos) );

   if(haveLDAPServer)
   {
       /* Set up the MQAIR for LDAP usage */
       AuthRec.AuthInfoType = MQAIT_CRL_LDAP;
       strncpy( AuthRec.AuthInfoConnName, LDAPServer, strlen(LDAPServer) );
   
       AuthRec.LDAPUserNamePtr = LDAPUser;
       AuthRec.LDAPUserNameLength = strlen(LDAPUser);
       strncpy( AuthRec.LDAPPassword, LDAPPwd, strlen(LDAPPwd) );
   
       ConnSCO.AuthInfoRecCount = 1;
       ConnSCO.AuthInfoRecOffset = 0;
       ConnSCO.AuthInfoRecPtr = &AuthRec;
   }    
   
   /* Set up MQSCO within the MQCNO */
   ConnOpt.SSLConfigPtr = &ConnSCO;
   printf( "MQSCO strucId '%.4s', version %d\n", ConnSCO.StrucId, ConnSCO.Version );

   printf( "Using cipher '%s', repository '%s'\n", ClientConn.SSLCipherSpec,
           ConnSCO.KeyRepository );

   printf( "Issuing MQCONNX call...\n" );
   
   MQCONNX(QMName,                 /* queue manager                  */
           &ConnOpt,               /* options for connection         */
           &Hcon,                  /* connection handle              */
           &CompCode,              /* completion code                */
           &CReason);              /* reason code                    */

   printf("MQCONNX returned reason code %d\n", CReason);

   /******************************************************************/
   /*                                                                */
   /*   Open the target message queue for output                     */
   /*                                                                */
   /******************************************************************/
   if(!CReason)
   {
     O_options = MQOO_OUTPUT            /* open queue for output     */
               | MQOO_FAIL_IF_QUIESCING /* but not if MQM stopping   */
               ;                        /* = 0x2010 = 8208 decimal   */
  
     printf("Opening queue: '%s'\n", QName);

     strncpy(ObjDesc.ObjectName, QName, strlen(QName));
     ObjDesc.ObjectType = MQOT_Q;

     MQOPEN(Hcon,                      /* connection handle            */
            &ObjDesc,                  /* object descriptor for queue  */
            O_options,                 /* open options                 */
            &hQueue,                   /* object handle                */
            &CompCode,                 /* MQOPEN completion code       */
            &Reason);                  /* reason code                  */
  
     /* report reason, if any; stop if failed      */
     if (Reason != MQRC_NONE)
     {
       printf("MQOPEN ended with reason code %d\n", Reason);
     } else {
       printf("Queue is now open - type one message per line (blank line to exit)\n");
     }
  
     if (CompCode == MQCC_FAILED)
     {
       printf("unable to open queue for output\n");
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Read lines from the file and put them to the message queue   */
   /*   Loop until null line or end of file, or there is a failure   */
   /*                                                                */
   /******************************************************************/
   fp = stdin;

   memcpy(md.Format,           /* character string format            */
          MQFMT_STRING, (size_t)MQ_FORMAT_LENGTH);

   while (CompCode != MQCC_FAILED)
   {
     if (fgets(buffer, sizeof(buffer), fp) != NULL)
     {
       messlen = (MQLONG)strlen(buffer);       /* length without null        */
       if (buffer[messlen-1] == '\n')  /* last char is a new-line    */
       {
         buffer[messlen-1]  = '\0';    /* replace new-line with null */
         --messlen;                    /* reduce buffer length       */
       }
     }
     else messlen = 0;        /* treat EOF same as null line         */

     /****************************************************************/
     /*                                                              */
     /*   Put each buffer to the message queue                       */
     /*                                                              */
     /****************************************************************/
     if (messlen > 0)
     {
       memcpy(md.MsgId,           /* reset MsgId to get a new one    */
              MQMI_NONE, sizeof(md.MsgId) );

       memcpy(md.CorrelId,        /* reset CorrelId to get a new one */
              MQCI_NONE, sizeof(md.CorrelId) );

       MQPUT(Hcon,                /* connection handle               */
             hQueue,                /* object handle                   */
             &md,                 /* message descriptor              */
             &pmo,                /* default options (datagram)      */
             messlen,             /* message length                  */
             buffer,              /* message buffer                  */
             &CompCode,           /* completion code                 */
             &Reason);            /* reason code                     */

       /* report reason, if any */
       if (Reason != MQRC_NONE)
       {
         printf("MQPUT ended with reason code %d\n", Reason);
       }
     }
     else   /* satisfy end condition when empty line is read */
       CompCode = MQCC_FAILED;
   }

   /******************************************************************/
   /*                                                                */
   /*   Disconnect from MQM if connected                             */
   /*                                                                */
   /******************************************************************/
   if (CReason == 0 )
   {
     MQDISC(&Hcon,                   /* connection handle            */
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("MQDISC ended with reason code %d\n", Reason);
     }
   }

   return(0);
 }
