#!/bin/sh 
#------------------------------------------------------------------------------#
# Licensed Materials - Property of IBM                                         #
#                                                                              #
# SupportPac MQ01                                                              #
#                                                                              #
#  Copyright IBM Corporation 2006                                             #
# All rights reserved                                                          #
#                                                                              #
# US Government Users Restricted Rights - Use, duplication or                  #
# disclosure restricted by GSA ADP Schedule Contract with                      #
# IBM Corp.                                                                    #
#------------------------------------------------------------------------------#
#                                                                              #
#---------------------------------------------------------------------------
# Example configuration script for SSL/LDAP 
#---------------------------------------------------------------------------
# This script makes a very simple 3-queue manager configuration
# on the local system. Intended for WebSphere MQ version 5.3 for Unix
# only. This script will not work with WebSphere MQ version 6.
#
# Function:
#   Creates queue managers SSL1, SSL2 and SSL3.
#   Starts the queue managers and a listener for each.
#   Configures SSL certificates (SSL1a, SSL2b and SSL3b respectively).
#   Defines some sender/receiver channel pairs between the queue managers.
#
# This means that the certificate for SSL1 is revoked and the others 
# are not. Therefore the channels to SSL1 will fail to start, whilst the 
# others will work.
#
# Most of the script contains error checking - the most relevant
# section is clearly marked below.
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# Subroutines to report errors 
#---------------------------------------------------------------------------
dir_error()
{
  echo 
  echo Please run this script from a directory containing root.crt and the
  echo PKCS12 .p12 files for this scenario.
  exit 1;
}

create_error()
{
  echo 
  echo An error occurred creating queue manager $QM_CURRENT.
  echo Please investigate the cause and fix it.
  exit 1;
}

start_error()
{
  echo 
  echo An error occurred starting queue manager $QM_CURRENT.
  echo Please investigate the cause and fix it.
  exit 1;
}

listener_error()
{
  echo 
  echo An error occurred configuring listeners for $QM_CURRENT.
  echo Unable to start a listener on port $QM_LPORT.
  exit 1;
}

usage()
{
  echo 
  echo Usage: setupv6 LDAPServer [LDAPPort]
  echo 
  echo Where:  LDAPServer is the TCP/IP host name of the LDAP server. This
  echo         server must have the supplied config.ldif file imported into its
  echo         directory.
  echo 
  echo         LDAPPort is the TCP/IP port number of the LDAP server. Defaults
  echo         to 389 if omitted.
  echo 
  exit 1
}

#------------------------------------------------------------------------------
# Validate parameters. Ensure all required files are present.
#------------------------------------------------------------------------------
if [ -z "$1" ]; then usage; fi  

WMQDIR=/var/mqm
LDAPSERVER=$1
LDAPPORT=$2
if [ -z "$LDAPPORT" ]; then LDAPPORT=389; fi

if [ ! -f root.crt ]; then dir_error; fi
if [ ! -f SSL1a.p12 ]; then dir_error; fi
if [ ! -f SSL2b.p12 ]; then dir_error; fi
if [ ! -f SSL3b.p12 ]; then dir_error; fi

# Locate a suitable echo command:
for a in /bin/echo /usr/bin/echo ; do [ -x $a ] && ECHO=$a ; done

#------------------------------------------------------------------------------
# SSL configuration begins here 
#------------------------------------------------------------------------------
echo 
echo ------------------------------------------------------------------------------
echo Creating queue managers on the local system...
echo ------------------------------------------------------------------------------
echo 

QM_CURRENT=SSL1
crtmqm $QM_CURRENT
if [ $? -gt 0 ]; then create_error; fi

QM_CURRENT=SSL2
crtmqm $QM_CURRENT
if [ $? -gt 0 ]; then create_error; fi

QM_CURRENT=SSL3
crtmqm $QM_CURRENT
if [ $? -gt 0 ]; then create_error; fi

echo 
echo ------------------------------------------------------------------------------
echo Queue managers created. Starting queue managers...
echo ------------------------------------------------------------------------------
echo 

QM_CURRENT=SSL1
strmqm $QM_CURRENT
if [ $? -gt 0 ]; then start_error; fi

QM_CURRENT=SSL2
strmqm $QM_CURRENT
if [ $? -gt 0 ]; then start_error; fi

QM_CURRENT=SSL3
strmqm $QM_CURRENT
if [ $? -gt 0 ]; then start_error; fi

echo 
echo ------------------------------------------------------------------------------
echo Queue managers started. Configuring listeners...
echo ------------------------------------------------------------------------------
echo 

QM_CURRENT=SSL1
QM_LPORT=1515
runmqlsr -m $QM_CURRENT -t tcp -p $QM_LPORT &
if [ $? -gt 0 ]; then listener_error; fi

QM_CURRENT=SSL2
QM_LPORT=1616
runmqlsr -m $QM_CURRENT -t tcp -p $QM_LPORT &
if [ $? -gt 0 ]; then listener_error; fi

QM_CURRENT=SSL3
QM_LPORT=1717
runmqlsr -m $QM_CURRENT -t tcp -p $QM_LPORT &
if [ $? -gt 0 ]; then listener_error; fi

echo 
echo ------------------------------------------------------------------------------
echo Creating key repositories...
echo ------------------------------------------------------------------------------
echo 
gsk6cmd -keydb -create -db $WMQDIR/qmgrs/SSL1/ssl/key.kdb -pw ldaptest -stash
if [ $? -gt 0 ]; then exit 1 ; fi

gsk6cmd -keydb -create -db $WMQDIR/qmgrs/SSL2/ssl/key.kdb -pw ldaptest -stash
if [ $? -gt 0 ]; then exit 1 ; fi

gsk6cmd -keydb -create -db $WMQDIR/qmgrs/SSL3/ssl/key.kdb -pw ldaptest -stash
if [ $? -gt 0 ]; then exit 1 ; fi

echo 
echo ------------------------------------------------------------------------------
echo Setting up each key repository...
echo ------------------------------------------------------------------------------
echo 

# First we add the certificate authority certificate then we import the PKCS12 file.
gsk6cmd -cert -add -db $WMQDIR/qmgrs/SSL1/ssl/key.kdb -pw ldaptest -file root.crt -format ascii -label rootca
if [ $? -gt 0 ] ; then exit 1 ; fi
gsk6cmd -cert -import -db SSL1a.p12 -pw ldaptest -target $WMQDIR/qmgrs/SSL1/ssl/key.kdb -pw ldaptest -label ibmwebspheremqssl1 -target_pw ldaptest
if [ $? -gt 0 ] ; then exit 1 ; fi

gsk6cmd -cert -add -db $WMQDIR/qmgrs/SSL2/ssl/key.kdb -pw ldaptest -file root.crt -format ascii -label rootca
if [ $? -gt 0 ] ; then exit 1 ; fi
gsk6cmd -cert -import -db SSL2b.p12 -pw ldaptest -target $WMQDIR/qmgrs/SSL2/ssl/key.kdb -pw ldaptest -label ibmwebspheremqssl2 -target_pw ldaptest
if [ $? -gt 0 ] ; then exit 1 ; fi

gsk6cmd -cert -add -db $WMQDIR/qmgrs/SSL3/ssl/key.kdb -pw ldaptest -file root.crt -format ascii -label rootca
if [ $? -gt 0 ] ; then exit 1 ; fi
gsk6cmd -cert -import -db SSL3b.p12 -pw ldaptest -target $WMQDIR/qmgrs/SSL3/ssl/key.kdb -pw ldaptest -label ibmwebspheremqssl3 -target_pw ldaptest
if [ $? -gt 0 ] ; then exit 1 ; fi

echo 
echo ------------------------------------------------------------------------------
echo Configuring queue managers to contact the LDAP server...
echo ------------------------------------------------------------------------------
echo 

echo "DEFINE AUTHINFO(AUTHINFO.LDAP) AUTHTYPE(CRLLDAP) CONNAME('$LDAPSERVER($LDAPPORT)') REPLACE" > tmp.mqsc
echo "DEFINE NAMELIST(NAMELIST.LDAP) NAMES(AUTHINFO.LDAP) REPLACE" >> tmp.mqsc
echo "ALTER QMGR SSLCRLNL(NAMELIST.LDAP)" >> tmp.mqsc

cat tmp.mqsc | runmqsc SSL1
if [ $? -gt 0 ] ; then exit 1 ; fi
cat tmp.mqsc | runmqsc SSL2
if [ $? -gt 0 ] ; then exit 1 ; fi
cat tmp.mqsc | runmqsc SSL3
if [ $? -gt 0 ] ; then exit 1 ; fi

del tmp.mqsc

echo 
echo ------------------------------------------------------------------------------
echo Defining example receiver channels...
echo ------------------------------------------------------------------------------
echo 

echo "DEFINE CHANNEL(TO.SSL1) CHLTYPE(RCVR) TRPTYPE(TCP) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL1
echo "DEFINE CHANNEL(TO.SSL2) CHLTYPE(RCVR) TRPTYPE(TCP) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL2
echo "DEFINE CHANNEL(TO.SSL3) CHLTYPE(RCVR) TRPTYPE(TCP) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL3

echo 
echo ------------------------------------------------------------------------------
echo Defining example transmission queues...
echo ------------------------------------------------------------------------------
echo 

echo "DEFINE QLOCAL(SSL2) USAGE(XMITQ) REPLACE " | runmqsc SSL1
echo "DEFINE QLOCAL(SSL3) USAGE(XMITQ) REPLACE " | runmqsc SSL1

echo "DEFINE QLOCAL(SSL1) USAGE(XMITQ) REPLACE " | runmqsc SSL2
echo "DEFINE QLOCAL(SSL3) USAGE(XMITQ) REPLACE " | runmqsc SSL2

echo "DEFINE QLOCAL(SSL1) USAGE(XMITQ) REPLACE " | runmqsc SSL3
echo "DEFINE QLOCAL(SSL2) USAGE(XMITQ) REPLACE " | runmqsc SSL3

echo  
echo ------------------------------------------------------------------------------
echo Defining example sender channels...
echo ------------------------------------------------------------------------------
echo 

# Listener ports are: SSL1=1515, SSL2=1616, SSL3=1717
echo "DEFINE CHANNEL(TO.SSL2) CHLTYPE(SDR) TRPTYPE(TCP) CONNAME('localhost(1616)') XMITQ(SSL2) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL1
echo "DEFINE CHANNEL(TO.SSL3) CHLTYPE(SDR) TRPTYPE(TCP) CONNAME('localhost(1717)') XMITQ(SSL3) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL1

echo "DEFINE CHANNEL(TO.SSL1) CHLTYPE(SDR) TRPTYPE(TCP) CONNAME('localhost(1515)') XMITQ(SSL1) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL2
echo "DEFINE CHANNEL(TO.SSL3) CHLTYPE(SDR) TRPTYPE(TCP) CONNAME('localhost(1717)') XMITQ(SSL3) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL2

echo "DEFINE CHANNEL(TO.SSL1) CHLTYPE(SDR) TRPTYPE(TCP) CONNAME('localhost(1515)') XMITQ(SSL1) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL3
echo "DEFINE CHANNEL(TO.SSL2) CHLTYPE(SDR) TRPTYPE(TCP) CONNAME('localhost(1616)') XMITQ(SSL2) SSLCIPH(RC4_SHA_US) REPLACE " | runmqsc SSL3

echo "
---------------------------------------------------------------------------
SSL configuration complete.                                                
---------------------------------------------------------------------------
Three queue managers have been set up: SSL1, SSL2 and SSL2.                

Some channel pairs have been defined between the queue managers; these     
are named TO.SSL1, TO.SSL2 and TO.SSL3.                                    

Next steps:                                                                

  (1) If you have not already done so, import the supplied config.ldif     
      into your LDAP server on '$LDAPSERVER'.                              

  (2) Try starting the sender channels - channels to SSL1 should fail      
      to start because SSL1's certificate is revoked.                       

Check each queue manager's error logs to determine the reason why each     
channel fails to start.                                                    
---------------------------------------------------------------------------"


exit 0;

