#!/bin/sh 
#------------------------------------------------------------------------------#
# Licensed Materials - Property of IBM                                         #
#                                                                              #
# SupportPac MQ01                                                              #
#                                                                              #
#  Copyright IBM Corporation 2006                                             #
# All rights reserved                                                          #
#                                                                              #
# US Government Users Restricted Rights - Use, duplication or                  #
# disclosure restricted by GSA ADP Schedule Contract with                      #
# IBM Corp.                                                                    #
#------------------------------------------------------------------------------#
# ---------------------------------------------------------------------------
# Example configuration script for SSL/LDAP 
# ---------------------------------------------------------------------------
# This script makes a very simple single queue manager configuration
# on the local system. Intended for WebSphere MQ version 6 for Unix
# only. This script will not work with WebSphere MQ version 5.3.
#
# Function:
#   Creates queue manager SALES
#   Starts the queue manager a listener for it.
#   Configures SSL certificates for SALES
#   Creates two alternative SSL key databases for the client to use
#   Defines a SVRCONN channel for the client to use
#
# The client1 key database contains a revoked client certificate, whereas 
# the client2 key database contains a valid client certificate.
#
# Most of the script contains error checking - the most relevant
# section is clearly marked below.
# ---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# Subroutines to report errors 
#---------------------------------------------------------------------------
dir_error()
{
  echo 
  echo Please run this script from a directory containing root.crt and the
  echo PKCS12 .p12 files for this scenario.
  exit 1;
}

create_error()
{
  echo 
  echo An error occurred creating queue manager SALES.
  echo Please investigate the cause and fix it.
  exit 1;
}

start_error()
{
  echo 
  echo An error occurred starting queue manager SALES.
  echo Please investigate the cause and fix it.
  exit 1;
}

listener_error()
{
  echo 
  echo An error occurred configuring listeners for SALES.
  echo Unable to start a listener on port $QM_LPORT.
  exit 1;
}

usage()
{
  echo 
  echo Usage: setupv6.sh LDAPServer [LDAPPort]
  echo 
  echo Where:  LDAPServer is the TCP/IP host name of the LDAP server. This
  echo         server must have the supplied config.ldif file imported into its
  echo         directory.
  echo 
  echo         LDAPPort is the TCP/IP port number of the LDAP server. Defaults
  echo         to 389 if omitted.
  echo 
  exit 1
}

#---------------------------------------------------------------------------
# Validate parameters. Ensure all required files are present.
#---------------------------------------------------------------------------
if [ -z "$1" ]; then usage; fi  

WMQDIR=/var/mqm
LDAPSERVER=$1
LDAPPORT=$2
if [ -z "$LDAPPORT" ]; then LDAPPORT=389; fi

if [ ! -f root.crt ]; then dir_error; fi
if [ ! -f SSL1a.p12 ]; then dir_error; fi
if [ ! -f SSL2b.p12 ]; then dir_error; fi
if [ ! -f SSL3b.p12 ]; then dir_error; fi

# Locate a suitable echo command
for a in /bin/echo /usr/bin/echo ; do [ -x $a ] && ECHO=$a ; done

#---------------------------------------------------------------------------
# SSL configuration begins here 
#---------------------------------------------------------------------------
echo 
echo ---------------------------------------------------------------------------
echo Creating queue managers on the local system...
echo ---------------------------------------------------------------------------
echo 

crtmqm SALES
if [ $? -gt 0 ]; then create_error; fi

echo 
echo ---------------------------------------------------------------------------
echo Queue managers created. Starting queue managers...
echo ---------------------------------------------------------------------------
echo 

strmqm SALES
if [ $? -gt 0 ]; then start_error; fi

echo 
echo ---------------------------------------------------------------------------
echo Queue managers started. Configuring listeners...
echo ---------------------------------------------------------------------------
echo 

QM_LPORT=1414
echo "DEFINE LISTENER(L1) TRPTYPE(TCP) PORT($QM_LPORT) " | runmqsc SALES
if [ $? -gt 0 ]; then listener_error; fi
echo "START LISTENER(L1) " | runmqsc SALES
if [ $? -gt 0 ]; then listener_error; fi

echo 
echo ------------------------------------------------------------------------------
echo Creating key repositories...
echo ------------------------------------------------------------------------------
echo 
gsk7cmd -keydb -create -db $WMQDIR/qmgrs/SALES/ssl/key.kdb -pw ldaptest -stash
if [ $? -gt 0 ]; then exit 1 ; fi

gsk7cmd -keydb -create -db client1.kdb -pw ldaptest -stash
if [ $? -gt 0 ]; then exit 1 ; fi

gsk7cmd -keydb -create -db client2.kdb -pw ldaptest -stash
if [ $? -gt 0 ]; then exit 1 ; fi

echo 
echo ------------------------------------------------------------------------------
echo Setting up each key repository...
echo ------------------------------------------------------------------------------
echo 

# First we add the certificate authority certificate then we import the PKCS12 file.
gsk7cmd -cert -add -db $WMQDIR/qmgrs/SALES/ssl/key.kdb -pw ldaptest -file root.crt -format ascii -label rootca
if [ $? -gt 0 ] ; then exit 1 ; fi
gsk7cmd -cert -import -db SALES.p12 -pw ldaptest -target $WMQDIR/qmgrs/SALES/ssl/key.kdb -pw ldaptest -label ibmwebspheremqsales -target_pw ldaptest
if [ $? -gt 0 ] ; then exit 1 ; fi

gsk7cmd -cert -add -db client1.kdb -pw ldaptest -file root.crt -format ascii -label rootca
if [ $? -gt 0 ] ; then exit 1 ; fi
gsk7cmd -cert -import -db client1.p12 -pw ldaptest -target client1.kdb -pw ldaptest -label ibmwebspheremqfred -target_pw ldaptest
if [ $? -gt 0 ] ; then exit 1 ; fi

gsk7cmd -cert -add -db client2.kdb -pw ldaptest -file root.crt -format ascii -label rootca
if [ $? -gt 0 ] ; then exit 1 ; fi
gsk7cmd -cert -import -db client2.p12 -pw ldaptest -target client2.kdb -pw ldaptest -label ibmwebspheremqfred -target_pw ldaptest
if [ $? -gt 0 ] ; then exit 1 ; fi

echo 
echo ------------------------------------------------------------------------------
echo Configuring queue managers to contact the LDAP server...
echo ------------------------------------------------------------------------------
echo 

echo "DEFINE AUTHINFO(AUTHINFO.LDAP) AUTHTYPE(CRLLDAP) CONNAME('$LDAPSERVER($LDAPPORT)') REPLACE" > tmp.mqsc
echo "DEFINE NAMELIST(NAMELIST.LDAP) NAMES(AUTHINFO.LDAP) REPLACE" >> tmp.mqsc
echo "ALTER QMGR SSLCRLNL(NAMELIST.LDAP)" >> tmp.mqsc

cat tmp.mqsc | runmqsc SALES
if [ $? -gt 0 ] ; then exit 1 ; fi

rm tmp.mqsc

echo "
---------------------------------------------------------------------------
SSL configuration complete. 
---------------------------------------------------------------------------
The SALES queue manager has been created and the SYSTEM.DEF.SVRCONN channel
has been configured for SSL connections.

Also, two client key repositories have been created: client1 and client2. 
These files are needed by the client to establish its SSL connection. The
client1 repository contains a revoked client certificate, whereas the other
contains a valid client certificate.

Next steps:

  (1) If you have not already done so, import the supplied config.ldif
      into your LDAP server on '$LDAPSERVER'.

  (2) Copy the client1 and client2 files to the client system. Ensure
      that the files are transferred in binary mode if FTP is used.

  (3) Try starting a client to connect to SALES on port $QM_LPORT using
      the SYSTEM.DEF.SVRCONN channel. This should fail when the client1
      key repository is used because that contains a revoked certificate.

Check the WebSphere MQ error logs to determine the reason why the 
channel fails to start.
---------------------------------------------------------------------------"

exit 0;
